@@ -30,6 +30,9 @@
 #include "FileSet.h"
 #include "ArgV.h"
 #include "ProtoLog.h"
+#include "xlist.h"
+#include "xmap.h"
+#include "Timer.h"
 
 #define FILE_END     ((off_t)-1L)
 #define UNKNOWN_POS  ((off_t)-1L)
@@ -39,6 +42,7 @@ class Glob;
 class NoGlob;
 class DirList;
 class FileAccessRef;
+class Buffer;
 
 class FileAccess : public SMTask, public ResClient, protected ProtoLog
 {
@@ -110,7 +114,7 @@ protected:
    const char *default_cwd;
    Path	 home;
    Path	 cwd;
-   Path	 *new_cwd;
+   Ref<Path> new_cwd;
    xstring file;
    xstring file_url;
    xstring file1;
@@ -124,7 +128,7 @@ protected:
 
    FileSet *fileset_for_info;
 
-   time_t try_time;
+   Timer reconnect_timer;
    int retries;
 
    bool	 mkdir_p;
@@ -169,8 +173,11 @@ protected:
    xstring_c entity_content_type;
    xstring_c entity_charset;
 
-   FileAccess *next;
-   static FileAccess *chain;
+   xstring_c last_disconnect_cause;
+
+   xlist<FileAccess> all_fa_node;
+   static xlist_head<FileAccess> all_fa;
+
    FileAccess *FirstSameSite() const { return NextSameSite(0); }
    FileAccess *NextSameSite(FileAccess *) const;
 
@@ -178,6 +185,8 @@ protected:
 
    int device_prefix_len(const char *path) const;
 
+   virtual ~FileAccess();
+
 public:
    virtual const char *GetProto() const = 0; // http, ftp, file etc
    bool SameProtoAs(const FileAccess *fa) const { return !strcmp(GetProto(),fa->GetProto()); }
@@ -194,6 +203,7 @@ public:
    const char  *GetConnectURL(int flags=0) const;
    const char  *GetFileURL(const char *file,int flags=0) const;
    enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4,NO_USER=8 };
+   const char *GetLastDisconnectCause() const { return last_disconnect_cause; }
 
    void Connect(const char *h,const char *p);
    void ConnectVerify();
@@ -231,7 +241,7 @@ public:
 
    virtual const char *CurrentStatus();
 
-   virtual int Read(void *buf,int size) = 0;
+   virtual int Read(Buffer *buf,int size) = 0;
    virtual int Write(const void *buf,int size) = 0;
    virtual int Buffered();
    virtual int StoreStatus() = 0;
@@ -253,18 +263,18 @@ public:
    bool IsBetterThan(const FileAccess *fa) const;
 
    void Init();
-   FileAccess() { Init(); }
+   FileAccess() : all_fa_node(this) { Init(); }
    FileAccess(const FileAccess *);
-   virtual ~FileAccess();
 
-   void	 DontSleep() { try_time=0; }
+   void	 DontSleep() { reconnect_timer.Stop(); }
 
    bool	 IsClosed() { return mode==CLOSED; }
    bool	 IsOpen() { return !IsClosed(); }
    int	 OpenMode() { return mode; }
 
    virtual int  IsConnected() const; // level of connection (0 - not connected).
-   virtual void Disconnect();
+   void Disconnect(const char *dc=0) { last_disconnect_cause.set(dc); DisconnectLL(); }
+   virtual void DisconnectLL() {}
    virtual void UseCache(bool);
    virtual bool NeedSizeDateBeforehand();
 
@@ -297,7 +307,7 @@ public:
    virtual DirList *MakeDirList(ArgV *a);
    virtual FileSet *ParseLongList(const char *buf,int len,int *err=0) const { return 0; }
 
-   static bool NotSerious(int err);
+   static bool NotSerious(int err) { return temporary_network_error(err); }
 
    const char *GetNewLocation() { return location; }
    const char *GetSuggestedFileName() { return suggested_filename; }
@@ -310,8 +320,7 @@ public:
    typedef FileAccess *SessionCreator();
    class Protocol
    {
-      static Protocol *chain;
-      Protocol *next;
+      static xmap<Protocol*> proto_by_name;
       const char *proto;
       SessionCreator *New;
 
@@ -346,7 +355,7 @@ public:
    // not pretty (FIXME)
    int GetRetries() { return retries; }
    void SetRetries(int r) { retries=r; }
-   time_t GetTryTime() { return try_time; }
+   time_t GetTryTime() { return reconnect_timer.GetStartTime(); }
    void SetTryTime(time_t t);
 
    const char *GetLogContext() { return hostname; }
@@ -463,10 +472,10 @@ protected:
    bool follow_symlinks;
 
    void PrepareToDie();
+   ~ListInfo();
 
 public:
    ListInfo(FileAccess *session,const char *path);
-   ~ListInfo();
 
    void SetExclude(const char *p,const PatternSet *e) { exclude_prefix=p; exclude=e; }
 
@@ -499,10 +508,10 @@ protected:
    Ref<Buffer> buf;
    Ref<ArgV> args;
    bool color;
+   ~DirList();
 
 public:
    DirList(FileAccess *s,ArgV *a);
-   ~DirList();
 
    virtual int Do() = 0;
    virtual const char *Status() = 0;
