@@ -54,6 +54,15 @@ ResDecl eta_period   ("xfer:eta-period", "120",ResMgr::UNumberValidate,ResMgr::N
 ResDecl max_redir    ("xfer:max-redirections", "5",ResMgr::UNumberValidate,ResMgr::NoClosure);
 ResDecl buffer_size  ("xfer:buffer-size","0x10000",ResMgr::UNumberValidate,ResMgr::NoClosure);
 
+enum {
+   // Delays in microseconds
+   MAX_DELAY=30000,
+   DELAY_STEP=30,
+   // This size is related to socket buffer size.
+   // When it is too large, tcp slowdown happens.
+   MAX_READ_TO_DELAY=0x4000,
+};
+
 // FileCopy
 #define super SMTask
 
@@ -638,6 +647,15 @@ void FileCopy::LogTransfer()
       const char *fname = ResMgr::Query("xfer:log-file", 0);
       if(!fname || !*fname)
 	 fname = dir_file(get_lftp_data_dir(),"transfer_log");
+      struct stat st;
+      if(stat(fname,&st)!=-1) {
+	 if(st.st_size > long(ResMgr::Query("xfer:max-log-size",0))) {
+	    debug((9,"rotating xfer-log %s",fname));
+	    if(rename(fname,xstring::cat(fname,".old",NULL))==-1) {
+	       debug((1,"rename(%s): %s",fname,strerror(errno)));
+	    }
+	 }
+      }
       int fd = open(fname,O_WRONLY|O_APPEND|O_CREAT,0600);
       if(fd==-1)
 	 return;
@@ -920,7 +938,7 @@ int FileCopyPeerFA::Do()
 	 return m;
       if(fxp)
 	 return m;
-      res=Get_LL(GET_BUFSIZE);
+      res=TuneGetSize(Get_LL(get_size));
       if(res>0)
       {
 	 EmbraceNewData(res);
@@ -1075,6 +1093,13 @@ void FileCopyPeerFA::RemoveFile()
 
 int FileCopyPeerFA::Get_LL(int len)
 {
+   if(get_delay>0)
+   {
+      if(!get_ll_timer.Stopped())
+	 return 0;
+      session->ResumeSlave();
+   }
+
    int res=0;
 
    if(session->IsClosed())
@@ -1087,7 +1112,7 @@ int FileCopyPeerFA::Get_LL(int len)
    if(GetRealPos()!=io_at) // GetRealPos can alter pos.
       return 0;
 
-   res=session->Read(GetSpace(len),len);
+   res=session->Read(this,len);
    if(res<0)
    {
       if(res==FA::DO_AGAIN)
@@ -1168,13 +1193,22 @@ int FileCopyPeerFA::Get_LL(int len)
       SetError(session->StrError(res));
       return -1;
    }
-   if(res==0)
+   else if(res==0)
    {
       eof=true;
       FileAccess::cache->Add(session,file,FAmode,FA::OK,this);
       SetSuggestedFileName(session->GetSuggestedFileName());
       session->Close();
    }
+   else if(res<=MAX_READ_TO_DELAY)
+   {
+      if(get_delay<=MAX_DELAY-DELAY_STEP)
+	 get_delay+=DELAY_STEP;
+      get_ll_timer.SetMicroSeconds(get_delay);
+      session->SuspendSlave();
+   }
+   else if(res>MAX_READ_TO_DELAY && get_delay>=DELAY_STEP)
+      get_delay-=DELAY_STEP;
    return res;
 }
 
@@ -1256,6 +1290,7 @@ off_t FileCopyPeerFA::GetRealPos()
 
 void FileCopyPeerFA::Init()
 {
+   get_delay=0;
    fxp=false;
    try_time=NO_DATE;
    retries=-1;
@@ -1443,7 +1478,7 @@ int FileCopyPeerFDStream::Do()
 	 {
 	    const char *new_name=dir_file(dirname(stream->full_name),suggested_filename);
 	    struct stat st;
-	    if(lstat(new_name,&st)==-1 && errno==ENOENT) {
+	    if((lstat(new_name,&st)==-1 && errno==ENOENT) || ResMgr::QueryBool("xfer:clobber",0)) {
 	       debug((5,"copy: renaming `%s' to `%s'\n",stream->full_name.get(),suggested_filename.get()));
 	       if(rename(stream->full_name,new_name)==-1)
 		  debug((3,"rename(%s, %s): %s\n",stream->full_name.get(),new_name,strerror(errno)));
@@ -1460,6 +1495,7 @@ int FileCopyPeerFDStream::Do()
    if(ascii)
       check_min_size=false;
 #endif
+   int res;
    switch(mode)
    {
    case PUT:
@@ -1491,43 +1527,29 @@ int FileCopyPeerFDStream::Do()
 	 return m;
       if(getfd()==-1)
 	 return m;
-      while(Size()>0)
-      {
-	 if(check_min_size && !eof && Size()<PUT_LL_MIN
-	 && put_ll_timer && !put_ll_timer->Stopped())
-	    break;
-	 int res=Put_LL(buffer+buffer_ptr,Size());
-	 if(res>0)
-	 {
-	    buffer_ptr+=res;
-	    m=MOVED;
-	 }
-	 if(res<0)
-	    return MOVED;
-	 if(res==0)
-	    break;
-      }
+
+      if(check_min_size && !eof && Size()<PUT_LL_MIN
+      && put_ll_timer && !put_ll_timer->Stopped())
+	 break;
+      res=Put_LL(buffer+buffer_ptr,Size());
+      if(res>0)
+	 buffer_ptr+=res;
+      if(res!=0)
+	 m=MOVED;
       break;
 
    case GET:
       if(eof)
 	 return m;
-      while(Size()<GET_BUFSIZE)
+
+      res=TuneGetSize(Get_LL(get_size));
+      if(res>0)
       {
-	 int res=Get_LL(GET_BUFSIZE);
-	 if(res>0)
-	 {
-	    EmbraceNewData(res);
-	    SaveMaxCheck(0);
-	    m=MOVED;
-	 }
-	 if(res<0)
-	    return MOVED;
-	 if(eof)
-	    return MOVED;
-	 if(res==0)
-	    break;
+	 EmbraceNewData(res);
+	 SaveMaxCheck(0);
       }
+      if(res!=0 || eof)
+	 m=MOVED;
       break;
    }
    return m;
