@@ -212,6 +212,7 @@ public:
    void Kill(int sig);
    off_t GetBytesCount() { return bytes_count; }
    double GetTimeSpent();
+   double GetTransferRate() { return rate.Get(); }
 
    void SetDate(time_t t,int p=0) { get->SetDate(t,p); }
    void SetDate(const FileTimestamp &t) { SetDate(t.ts,t.ts_prec); }
@@ -290,6 +291,10 @@ class FileCopyPeerFA : public FileCopyPeer
    int Put_LL(const char *buf,int size);
    int PutEOF_LL();
 
+   // to read data in larger quantities, delay the read op
+   Timer get_ll_timer;
+   int get_delay;
+
    FileSet info;
 
    bool fxp;   // FXP (ftp<=>ftp copy) active
