@@ -89,6 +89,7 @@ public:
    ~FileInfo();
 
    void SetName(const char *n) { name.set(n); def(NAME); }
+   void SetName(const xstring& n) { name.set(n); def(NAME); }
    void SetUser(const char *n);
    void SetGroup(const char *n);
    void LocalFile(const char *name, bool follow_symlinks);
@@ -109,6 +110,7 @@ public:
    bool	 NotOlderThan(time_t t) const;
    bool	 NotNewerThan(time_t t) const;
    bool  SizeOutside(const Range *r) const;
+   bool	 TypeIs(type t) const { return (defined&TYPE) && filetype==t; }
 
    void	 SetAssociatedData(const void *d,int len) { data.nset((const char*)d,len); }
    const void *GetAssociatedData() const { return data; }
