@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -249,10 +249,7 @@ int Fish::Do()
       }
       if(RespQueueSize()==0)
       {
-	 if(mode==ARRAY_INFO && array_ptr<array_cnt)
-	    SendArrayInfoRequests();
-	 else
-	    state=DONE;
+	 state=DONE;
 	 m=MOVED;
       }
       break;
@@ -304,9 +301,9 @@ void Fish::MoveConnectionHere(Fish *o)
    ResumeInternal();
 }
 
-void Fish::Disconnect()
+void Fish::DisconnectLL()
 {
-   super::Disconnect();
+   super::DisconnectLL();
    EmptyRespQueue();
    EmptyPathQueue();
    state=DISCONNECTED;
@@ -344,10 +341,13 @@ void Fish::Close()
    switch(state)
    {
    case(DISCONNECTED):
-   case(WAITING):
    case(CONNECTED):
    case(DONE):
       break;
+   case(WAITING):
+      if(mode==STORE || mode==RETRIEVE)
+	 Disconnect();
+      break;
    case(FILE_SEND):
       if(!RespQueueIsEmpty())
 	 Disconnect();
@@ -378,22 +378,16 @@ void Fish::Send(const char *format,...)
 
 void Fish::SendArrayInfoRequests()
 {
-   for(int i=array_ptr; i<array_cnt; i++)
+   for(int i=fileset_for_info->curr_index(); i<fileset_for_info->count(); i++)
    {
-      if(array_for_info[i].get_time || array_for_info[i].get_size)
+      FileInfo *fi=(*fileset_for_info)[i];
+      if(fi->need)
       {
-	 const char *e=shell_encode(array_for_info[i].file);
+	 const char *e=shell_encode(fi->name);
 	 Send("#INFO %s\n"
-	      "ls -lLd %s; echo '### 200'\n",array_for_info[i].file,e);
+	      "ls -lLd %s; echo '### 200'\n",fi->name.get(),e);
 	 PushExpect(EXPECT_INFO);
       }
-      else
-      {
-	 if(i==array_ptr)
-	    array_ptr++;   // if it is first one, just skip it.
-	 else
-	    break;	   // otherwise, wait until it is first.
-      }
    }
 }
 
@@ -575,6 +569,10 @@ int Fish::HandleReplies()
 	 Disconnect();
 	 return MOVED;
       }
+      if(entity_size!=NO_SIZE && real_pos<entity_size)
+	 return m;
+      if(entity_size==NO_SIZE)
+	 return m;
    }
    recv_buf->Put(pty_recv_buf->Get(),pty_recv_buf->Size()); // join the messages.
    pty_recv_buf->Skip(pty_recv_buf->Size());
@@ -625,8 +623,10 @@ int Fish::HandleReplies()
    {
       if(message==0)
 	 message.set(line);
-      else
-	 message.vappend("\n",line.get(),NULL);
+      else {
+	 message.append('\n');
+	 message.append(line);
+      }
       return m;
    }
 
@@ -686,7 +686,7 @@ int Fish::HandleReplies()
       else if(message && message[0]!='#')
       {
 	 FileInfo *fi=FileInfo::parse_ls_line(message,"GMT");
-	 if(!fi)
+	 if(!fi || !strncmp(message,"ls: ",4))
 	 {
 	    SetError(NO_FILE,message);
 	    break;
@@ -708,18 +708,12 @@ int Fish::HandleReplies()
       break;
    case EXPECT_INFO:
    {
-      FileInfo *fi=FileInfo::parse_ls_line(message,"GMT");
-      if(fi && fi->defined&fi->SIZE)
-	 array_for_info[array_ptr].size=fi->size;
-      else
-	 array_for_info[array_ptr].size=NO_SIZE;
-      if(fi && fi->defined&fi->DATE)
-	 array_for_info[array_ptr].time=fi->date;
-      else
-	 array_for_info[array_ptr].time=NO_DATE;
-      array_for_info[array_ptr].get_size=false;
-      array_for_info[array_ptr].get_time=false;
-      array_ptr++;
+      Ref<FileInfo> new_info(FileInfo::parse_ls_line(message,"GMT"));
+      FileInfo *fi=fileset_for_info->curr();
+      while(!fi->need)
+	 fi=fileset_for_info->next();
+      fi->Merge(*new_info);
+      fi->need=0;
       break;
    }
    case EXPECT_RETR:
@@ -772,14 +766,14 @@ void Fish::CloseExpectQueue()
       case EXPECT_PWD:
       case EXPECT_CWD:
 	 break;
-      case EXPECT_RETR_INFO:
       case EXPECT_INFO:
-      case EXPECT_RETR:
       case EXPECT_DIR:
-      case EXPECT_QUOTE:
       case EXPECT_DEFAULT:
 	 RespQueue[i]=EXPECT_IGNORE;
 	 break;
+      case EXPECT_QUOTE:
+      case EXPECT_RETR_INFO:
+      case EXPECT_RETR:
       case EXPECT_STOR_PRELIMINARY:
       case EXPECT_STOR:
 	 Disconnect();
@@ -801,7 +795,7 @@ const char *memstr(const char *mem,size_t len,const char *str)
    return 0;
 }
 
-int Fish::Read(void *buf,int size)
+int Fish::Read(Buffer *buf,int size)
 {
    if(Error())
       return error_code;
@@ -877,8 +871,7 @@ int Fish::Read(void *buf,int size)
       }
       if(size>size1)
 	 size=size1;
-      memcpy(buf,buf1,size);
-      recv_buf->Skip(size);
+      buf->MoveDataHere(recv_buf,size);
       pos+=size;
       real_pos+=size;
       rate_limit->BytesGot(size);
@@ -963,6 +956,7 @@ int Fish::Done()
 
 void Fish::SuspendInternal()
 {
+   super::SuspendInternal();
    if(recv_buf)
       recv_buf->SuspendSlave();
    if(send_buf)
@@ -974,6 +968,7 @@ void Fish::ResumeInternal()
       recv_buf->ResumeSlave();
    if(send_buf)
       send_buf->ResumeSlave();
+   super::ResumeInternal();
 }
 
 const char *Fish::CurrentStatus()
@@ -1162,6 +1157,7 @@ const char *FishDirList::Status()
 
 void FishDirList::SuspendInternal()
 {
+   super::SuspendInternal();
    if(ubuf)
       ubuf->SuspendSlave();
 }
@@ -1169,6 +1165,7 @@ void FishDirList::ResumeInternal()
 {
    if(ubuf)
       ubuf->ResumeSlave();
+   super::ResumeInternal();
 }
 
 static FileSet *ls_to_FileSet(const char *b,int len)
