@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2014 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -210,7 +210,7 @@ void Http::MoveConnectionHere(Http *o)
    ResumeInternal();
 }
 
-void Http::Disconnect()
+void Http::DisconnectLL()
 {
    rate_limit=0;
    if(conn)
@@ -282,8 +282,8 @@ void Http::Close()
    else
    {
    disconnect:
-      try_time=0;
       Disconnect();
+      DontSleep();
    }
    array_send=0;
    no_cache_this=false;
@@ -368,6 +368,9 @@ void Http::SendMethod(const char *method,const char *efile)
       accept=Query("accept-charset",hostname);
       if(accept && accept[0])
 	 Send("Accept-Charset: %s\r\n",accept);
+      accept=Query("accept-encoding",hostname);
+      if(accept && accept[0])
+	 Send("Accept-Encoding: %s\r\n",accept);
 
       const char *referer=Query("referer",hostname);
       const char *slash="";
@@ -823,7 +826,7 @@ void Http::ProceedArrayInfo()
    else
    {
       Disconnect();
-      try_time=0;
+      DontSleep();
    }
 }
 
@@ -854,8 +857,12 @@ static const char *extract_quoted_header_value(const char *value)
 
 void Http::HandleHeaderLine(const char *name,const char *value)
 {
-   if(!strcasecmp(name,"Content-length"))
-   {
+   // use a perfect hash
+#define hh(L,C) ((L)+(C)*3)
+#define hhc(S,C) hh(sizeof((S))-1,(C))
+#define case_hh(S,C) case hhc((S),(C)): if(strcasecmp(name,(S))) break;
+   switch(hh(strlen(name),c_toupper(name[0]))) {
+   case_hh("Content-Length",'C') {
       long long bs=0;
       if(1!=sscanf(value,"%lld",&bs))
 	 return;
@@ -876,8 +883,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       }
       return;
    }
-   if(!strcasecmp(name,"Content-range"))
-   {
+   case_hh("Content-Range",'C') {
       long long first,last,fsize;
       if(H_REQUESTED_RANGE_NOT_SATISFIABLE(status_code))
       {
@@ -900,8 +906,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 *opt_size=fsize;
       return;
    }
-   if(!strcasecmp(name,"Last-Modified"))
-   {
+   case_hh("Last-Modified",'L') {
       if(!H_2XX(status_code))
 	 return;
 
@@ -920,19 +925,16 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       }
       return;
    }
-   if(!strcasecmp(name,"Location"))
-   {
+   case_hh("Location",'L')
       location.set(value);
       return;
-   }
-   if(!strcasecmp(name,"Retry-After"))
-   {
+
+   case_hh("Retry-After",'R')
       retry_after=0;
       sscanf(value,"%ld",&retry_after);
       return;
-   }
-   if(!strcasecmp(name,"Keep-Alive"))
-   {
+
+   case_hh("Keep-Alive",'K') {
       keep_alive=true;
       const char *m=strstr(value,"max=");
       if(m) {
@@ -942,17 +944,17 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 keep_alive_max=100;
       return;
    }
-   if(!strcasecmp(name,"Connection")
-   || !strcasecmp(name,"Proxy-Connection"))
-   {
+   case_hh("Proxy-Connection",'P')
+      goto case_Connection;
+   case_hh("Connection",'C')
+   case_Connection:
       if(!strcasecmp(value,"keep-alive"))
 	 keep_alive=true;
       else if(!strcasecmp(value,"close"))
 	 keep_alive=false;
       return;
-   }
-   if(!strcasecmp(name,"Transfer-Encoding"))
-   {
+
+   case_hh("Transfer-Encoding",'T')
       if(!strcasecmp(value,"identity"))
 	 return;
       if(!strcasecmp(value,"chunked"))
@@ -963,10 +965,13 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 chunked_trailer=false;
       }
       return;
-   }
-   if(!strcasecmp(name,"Content-Encoding"))
-   {
-      if(!strcasecmp(value,"gzip")) {
+
+   case_hh("Content-Encoding",'C')
+      if(!QueryBool("decode",hostname))
+	 return;
+      if(!strcmp(value,"deflate")
+      || !strcmp(value,"gzip") || !strcmp(value,"compress")
+      || !strcmp(value,"x-gzip") || !strcmp(value,"x-compress")) {
 	 // inflated size if unknown beforehand
 	 entity_size=NO_SIZE;
 	 if(opt_size)
@@ -975,23 +980,21 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 inflate=new DirectedBuffer(DirectedBuffer::GET);
 	 inflate->SetTranslator(new DataInflator());
       }
-   }
-   if(!strcasecmp(name,"Accept-Ranges"))
-   {
+      return;
+
+   case_hh("Accept-Ranges",'A')
       if(!strcasecmp(value,"none"))
 	 no_ranges=true;
       if(strstr(value,"bytes"))
 	 seen_ranges_bytes=true;
       return;
-   }
-   if(!strcasecmp(name,"Set-Cookie"))
-   {
+
+   case_hh("Set-Cookie",'S')
       if(!hftp && QueryBool("set-cookies",hostname))
 	 SetCookie(value);
       return;
-   }
-   if(!strcasecmp(name,"Content-Disposition"))
-   {
+
+   case_hh("Content-Disposition",'C') {
       const char *filename=strstr(value,"filename=");
       if(!filename)
 	 return;
@@ -999,8 +1002,7 @@ void Http::HandleHeaderLine(const char *name,const char *value)
       SetSuggestedFileName(filename);
       return;
    }
-   if(!strcasecmp(name,"Content-Type"))
-   {
+   case_hh("Content-Type",'C') {
       entity_content_type.set(value);
       const char *cs=strstr(value,"charset=");
       if(cs)
@@ -1008,7 +1010,12 @@ void Http::HandleHeaderLine(const char *name,const char *value)
 	 cs=extract_quoted_header_value(cs+8);
 	 entity_charset.set(cs);
       }
+      return;
    }
+   default:
+      break;
+   }
+   LogNote(10,"unhandled header line `%s'",name);
 }
 
 static
@@ -1075,8 +1082,7 @@ int Http::Do()
    }
 
    if(home.path==0)
-      home.Set(default_cwd);
-   ExpandTildeInCWD();
+      set_home(default_cwd);
 
    if(Error())
       return m;
@@ -1289,7 +1295,6 @@ int Http::Do()
       if(res==-1)
       {
 	 NextPeer();
-	 Disconnect();
 	 return MOVED;
       }
       if(!(res&POLLOUT))
@@ -1348,6 +1353,7 @@ int Http::Do()
 	 SetError(NOT_SUPP);
 	 return MOVED;
       }
+      ExpandTildeInCWD();
       if(mode==STORE && pos>0 && entity_size>=0 && pos>=entity_size)
       {
 	 state=DONE;
@@ -1459,8 +1465,8 @@ int Http::Do()
 		     {
 			ResMgr::Set("http:use-propfind",hostname,"no");
 			use_propfind_now=false;
-			try_time=0;
 			Disconnect();
+			DontSleep();
 			return MOVED;
 		     }
 		     goto pre_RECEIVING_BODY;
@@ -1554,7 +1560,7 @@ int Http::Do()
 		  {
 		     Disconnect();
 		     if(retry_after)
-			try_time = time_t(now)+retry_after-long(reconnect_interval_current);
+			reconnect_timer.StopDelayed(retry_after);
 		     return MOVED;
 		  }
 
@@ -1665,8 +1671,8 @@ int Http::Do()
 	       if(mode==CHANGE_DIR && !xstrcmp(last_method,"PROPFIND"))
 	       {
 		  use_propfind_now=false;
-		  try_time=0;
 		  Disconnect();
+		  DontSleep();
 		  return MOVED;
 	       }
 	       code=NOT_SUPP;
@@ -1684,8 +1690,8 @@ int Http::Do()
 	 {
 	    if(!H_TRANSIENT(status_code))
 	       fileset_for_info->next();
-	    try_time=0;
 	    Disconnect();
+	    DontSleep();
 	 }
 	 else
 	    SetError(code,err);
@@ -1798,7 +1804,7 @@ void Http::ResumeInternal()
    super::ResumeInternal();
 }
 
-int Http::Read(void *buf,int size)
+int Http::Read(Buffer *buf,int size)
 {
    if(Error())
       return error_code;
@@ -1822,20 +1828,26 @@ int Http::Read(void *buf,int size)
 }
 void Http::_Skip(int to_skip)
 {
-   if(inflate) {
+   if(inflate)
       inflate->Skip(to_skip);
-   } else {
+   else
       conn->recv_buf->Skip(to_skip);
+   _UpdatePos(to_skip);
+}
+void Http::_UpdatePos(int to_skip)
+{
+   if(!inflate) {
       if(chunked)
 	 chunk_pos+=to_skip;
       bytes_received+=to_skip;
    }
    real_pos+=to_skip;
 }
-int Http::_Read(void *buf,int size)
+int Http::_Read(Buffer *buf,int size)
 {
    const char *buf1;
    int size1;
+   Buffer *src_buf=conn->recv_buf.get_non_const();
 get_again:
    if(conn->recv_buf->Size()==0 && conn->recv_buf->Error())
    {
@@ -1949,6 +1961,7 @@ get_again:
 	 bytes_received+=size1;
       }
       inflate->Get(&buf1,&size1);
+      src_buf=inflate.get_non_const();
    } else {
       if(size1>bytes_allowed)
 	 size1=bytes_allowed;
@@ -1967,8 +1980,8 @@ get_again:
    }
    if(size>size1)
       size=size1;
-   memcpy(buf,buf1,size);
-   _Skip(size);
+   buf->MoveDataHere(src_buf,size);
+   _UpdatePos(size);
    return size;
 }
 
@@ -2290,7 +2303,7 @@ void Http::CookieMerge(xstring &all,const char *cookie_c)
 
 void Http::MakeCookie(xstring &all_cookies,const char *hostname,const char *efile)
 {
-   ResMgr::Resource *scan=0;
+   Resource *scan=0;
    const char *closure;
    for(;;)
    {
@@ -2445,11 +2458,13 @@ void Http::LogErrorText()
    int size=conn->recv_buf->Size();
    if(size==0)
       return;
-   char *buf=string_alloca(size+1);
-   size=_Read(buf,size);
-   if(size<0)
+   Buffer tmpbuf;
+   size=_Read(&tmpbuf,size);
+   if(size<=0)
       return;
-   buf[size]=0;
+   tmpbuf.SpaceAdd(size);
+   const char *buf0=tmpbuf.Get();
+   char *buf=alloca_strdup(buf0);
    remove_tags(buf);
    for(char *line=strtok(buf,"\n"); line; line=strtok(0,"\n")) {
       rtrim(line);
