@@ -17,8 +17,6 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: NetAccess.cc,v 1.112 2011/06/16 11:08:42 lav Exp $ */
-
 #include <config.h>
 
 #include <errno.h>
@@ -127,14 +125,16 @@ int NetAccess::CheckHangup(const struct pollfd *pfd,int num)
 	 return 0;
       if(errno!=0 || s_errno!=0)
       {
-	 LogError(0,_("Socket error (%s) - reconnecting"),
-				    strerror(errno?errno:s_errno));
+	 const char *error=strerror(errno?errno:s_errno);
+	 LogError(0,_("Socket error (%s) - reconnecting"),error);
+	 Disconnect(error);
 	 return 1;
       }
 #endif /* SO_ERROR */
       if(pfd[i].revents&POLLERR)
       {
 	 LogError(0,"POLLERR on fd %d",pfd[i].fd);
+	 Disconnect("POLLERR");
 	 return 1;
       }
    } /* end for */
@@ -243,7 +243,7 @@ void NetAccess::NextPeer()
    if(peer_curr>=peer.count())
       peer_curr=0;
    else
-      try_time=0;	// try next address immediately
+      DontSleep(); // try next address immediately
 }
 
 void NetAccess::ResetLocationData()
@@ -301,11 +301,8 @@ bool NetAccess::ReconnectAllowed()
       return true; // it will fault later - no need to wait.
    if(connection_limit>0 && connection_limit<=CountConnections())
       return false;
-   if(try_time==0)
+   if(reconnect_timer.Stopped())
       return true;
-   if(time_t(now) >= try_time+long(reconnect_interval_current))
-      return true;
-   TimeoutS(long(reconnect_interval_current)-(time_t(now)-try_time));
    return false;
 }
 
@@ -313,10 +310,12 @@ const char *NetAccess::DelayingMessage()
 {
    if(connection_limit>0 && connection_limit<=CountConnections())
       return _("Connection limit reached");
-   long remains=long(reconnect_interval_current)-(time_t(now)-try_time);
+   long remains=reconnect_timer.TimeLeft();
    if(remains<=0)
       return "";
    current->TimeoutS(1);
+   if(last_disconnect_cause && reconnect_timer.TimePassed()<5)
+      return last_disconnect_cause;
    return xstring::format("%s: %ld",_("Delaying before reconnect"),remains);
 }
 
@@ -342,7 +341,8 @@ bool NetAccess::CheckRetries()
       Fatal(_("max-retries exceeded"));
       return false;
    }
-   try_time=now;
+   reconnect_timer.Set(reconnect_interval_current);
+   LogNote(10,"attempt number %d",retries);
    return true;
 }
 void NetAccess::TrySuccess()
@@ -416,7 +416,21 @@ do_again:
    if(done)
       return m;
 
-   if(!ubuf && !get_info)
+   if(session->OpenMode()==FA::ARRAY_INFO)
+   {
+      res=session->Done();
+      if(res==FA::DO_AGAIN)
+	 return m;
+      if(res==FA::IN_PROGRESS)
+	 return m;
+      session->Close();
+      FileAccess::cache->UpdateFileSet(session,"",FA::MP_LIST,result);
+      FileAccess::cache->UpdateFileSet(session,"",FA::LONG_LIST,result);
+      done=true;
+      return MOVED;
+   }
+
+   if(!ubuf)
    {
       const char *cache_buffer=0;
       int cache_buffer_size=0;
@@ -523,19 +537,16 @@ got_fileset:
       if(!result)
 	 result=new FileSet;
 
-      result->ExcludeDots();
       result->Exclude(exclude_prefix,exclude);
       result->rewind();
       for(file=result->curr(); file!=0; file=result->next())
       {
-	 FA::fileinfo add;
-	 add.size=NO_SIZE;
-	 add.time=NO_DATE;
-	 add.get_size = need_size && !(file->defined & file->SIZE);
-	 add.get_time = need_time && (!(file->defined & file->DATE)
-				 || (file->date.ts_prec>0 && can_get_prec_time));
-	 add.is_dir=false;
-	 add.file=0;
+	 file->need=0;
+	 if(need_size && !file->Has(file->SIZE))
+	    file->Need(file->SIZE);
+	 if(need_time && (!file->Has(file->DATE)
+                          || (file->date.ts_prec>0 && can_get_prec_time)))
+	    file->Need(file->DATE);
 
 	 if(file->defined & file->TYPE)
 	 {
@@ -543,61 +554,27 @@ got_fileset:
 	    {
 	       //file->filetype=file->NORMAL;
 	       file->defined &= ~(file->SIZE|file->SYMLINK_DEF|file->MODE|file->DATE|file->TYPE);
-	       add.get_size=true;
-	       add.get_time=true;
+	       file->Need(file->SIZE|file->DATE);
 	    }
-
-	    if(file->filetype==file->SYMLINK)
+	    else if(file->filetype==file->SYMLINK)
 	    {
 	       // don't need these for symlinks
-	       add.get_size=false;
-	       add.get_time=false;
+	       file->NoNeed(file->SIZE|file->DATE);
+	       // but need the link target
+	       if(!file->Has(file->SYMLINK_DEF))
+		  file->Need(file->SYMLINK_DEF);
 	    }
 	    else if(file->filetype==file->DIRECTORY)
 	    {
 	       if(!get_time_for_dirs)
 		  continue;
 	       // don't need size for directories
-	       add.get_size=false;
-	       add.is_dir=true;
+	       file->NoNeed(file->SIZE);
 	    }
 	 }
-
-	 if(add.get_size || add.get_time)
-	 {
-	    add.file=file->name;
-	    if(!add.get_size)
-	       add.size=NO_SIZE;
-	    if(!add.get_time)
-	       add.time=NO_DATE;
-	    get_info.append(add);
-	 }
       }
-      if(get_info.count()==0)
-      {
-	 done=true;
-	 return m;
-      }
-      session->GetInfoArray(get_info.get_non_const(),get_info.count());
-   }
-   if(get_info)
-   {
-      res=session->Done();
-      if(res==FA::DO_AGAIN)
-	 return m;
-      if(res==FA::IN_PROGRESS)
-	 return m;
-      session->Close();
-      for(int i=0; i<get_info.count(); i++)
-      {
-	 if(get_info[i].time!=NO_DATE)
-	    result->SetDate(get_info[i].file,get_info[i].time,0);
-	 if(get_info[i].size!=NO_SIZE)
-	    result->SetSize(get_info[i].file,get_info[i].size);
-      }
-      get_info.unset();
-      done=true;
-      m=MOVED;
+      session->GetInfoArray(result.get_non_const());
+      session->Roll();
    }
    return m;
 }
@@ -616,7 +593,7 @@ const char *GenericParseListInfo::Status()
       return xstring::format("%s (%lld) %s[%s]",_("Getting directory contents"),
 		     (long long)session->GetPos(),
 		     ubuf->GetRateStrS(),session->CurrentStatus());
-   if(get_info)
+   if(session->OpenMode()==FA::ARRAY_INFO)
       return xstring::format("%s (%d%%) [%s]",_("Getting files information"),
 		     session->InfoArrayPercentDone(),
 		     session->CurrentStatus());
