@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1996-2013 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2014 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -154,10 +154,14 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
 
    xstring md_download;
    size_t metadata_size;
-   void StartMetadataDownload() { md_download.nset("",0); }
+   void FetchMetadataFromURL(const char *url);
+   void StartMetadataDownload();
    void MetadataDownloaded();
    void SetMetadata(const xstring& md);
    void ParseMagnet(const char *p);
+   const char *GetMetadataPath() const;
+   void SaveMetadata() const;
+   bool LoadMetadata(const char *path);
 
    xstring_c metainfo_url;
    SMTaskRef<FileCopy> metainfo_copy;
@@ -419,6 +423,8 @@ class TorrentPeer : public SMTask, protected ProtoLog, public Networker
    bool connected;
    bool passive;
 
+   xstring_c last_dc;
+
    Timer timeout_timer;
    Timer retry_timer;
    Timer keepalive_timer;
@@ -692,7 +698,7 @@ private:
    void SendHandshake();
    unpack_status_t RecvHandshake();
    void SendExtensions();
-   void Disconnect();
+   void Disconnect(const char *dc=0);
    int SendDataRequests(unsigned p);
    void SendDataRequests();
    void Have(unsigned p);
