@@ -211,9 +211,12 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
 	   )},
    {"connect", cmd_open,   0,"open"},
    {"command", cmd_command},
-   {"debug",   cmd_debug,  N_("debug [<level>|off] [-o <file>]"),
+   {"debug",   cmd_debug,  N_("debug [OPTS] [<level>|off]"),
 	 N_("Set debug level to given value or turn debug off completely.\n"
-	 " -o <file>  redirect debug output to the file.\n")},
+	 " -o <file>  redirect debug output to the file\n"
+	 " -c  show message context\n"
+	 " -p  show PID\n"
+	 " -t  show timestamps\n")},
    {"du",      cmd_du,  N_("du [options] <dirs>"),
 	 N_("Summarize disk usage.\n"
 	 " -a, --all             write counts for all files, not just directories\n"
@@ -674,6 +677,11 @@ Job *CmdExec::builtin_exit()
 	 if(top)
 	    exec=top.get_non_const();
       }
+      else if(!strcmp(a,"parent"))
+      {
+	 if(parent_exec)
+	    exec=parent_exec;
+      }
       else if(!strcmp(a,"kill"))
       {
 	 kill=true;
@@ -687,7 +695,7 @@ Job *CmdExec::builtin_exit()
    }
    // Note: one job is this CmdExec.
    if(!bg && exec->top_level
-   && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfJobs()>0)
+   && !ResMgr::QueryBool("cmd:move-background",0) && NumberOfChildrenJobs()>0)
    {
       eprintf(_(
 	 "There are running jobs and `cmd:move-background' is not set.\n"
@@ -695,13 +703,15 @@ Job *CmdExec::builtin_exit()
       ));
       return 0;
    }
-   if(!detach && Job::NumberOfJobs()==0)
+   if(!detach && Job::NumberOfChildrenJobs()==0)
       detach=true;
    if(kill)
       Job::KillAll();
-   if(detach)
+   if(detach) {
+      for(CmdExec *e=this; e!=exec; e=e->parent_exec)
+	 e->Exit(code);
       exec->Exit(code);
-   else {
+   } else {
       int loc=0;
       exec->SetAutoTerminateInBackground(true);
       eprintf(_(
@@ -733,6 +743,7 @@ void CmdExec::Exit(int code)
 {
    while(feeder)
       RemoveFeeder();
+   cmd_buf.Empty();
    if(interactive)
    {
       ListDoneJobs();
@@ -827,13 +838,15 @@ Job *CmdExec::builtin_open()
 
    enum {
       OPT_USER,
-      OPT_PASSWORD
+      OPT_PASSWORD,
+      OPT_ENV_PASSWORD,
    };
    static struct option open_options[]=
    {
       {"port",required_argument,0,'p'},
       {"user",required_argument,0,OPT_USER},
       {"password",required_argument,0,OPT_PASSWORD},
+      {"env-password",no_argument,0,OPT_ENV_PASSWORD},
       {"execute",required_argument,0,'e'},
       {"debug",optional_argument,0,'d'},
       {"no-bookmark",no_argument,0,'B'},
@@ -873,6 +886,9 @@ Job *CmdExec::builtin_open()
       case(OPT_PASSWORD):
 	 pass=optarg;
 	 break;
+      case(OPT_ENV_PASSWORD):
+	 pass=getenv("LFTP_PASSWORD");
+	 break;
       case('d'):
 	 debug=true;
 	 break;
@@ -2335,7 +2351,7 @@ CMD(wait)
       eprintf(_("%s: wait loop detected\n"),op);
       return 0;
    }
-   j->parent=0;
+   j->SetParent(0);
    j->Bg();
    return j;
 }
