@@ -173,7 +173,7 @@ class Ftp : public NetAccess
       void SuspendInternal()
       {
 	 if(control_send) control_send->SuspendSlave();
-	 if(control_recv) control_recv->SuspendSlave();
+	 if(control_recv && !data_iobuf) control_recv->SuspendSlave();
 	 if(data_iobuf)	data_iobuf->SuspendSlave();
       }
       void ResumeInternal()
@@ -413,12 +413,16 @@ private:
 
    bool use_telnet_iac;
 
+   int max_buf;
+
    const char *get_protect_res();
    const char *encode_eprt(const sockaddr_u *);
 
    typedef FileInfo *(*FtpLineParser)(char *line,int *err,const char *tz);
    static FtpLineParser line_parsers[];
 
+   int CanRead();
+
 protected:
    ~Ftp();
 
@@ -455,7 +459,7 @@ public:
       return GetConnectLevel()!=CL_NOT_CONNECTED;
    }
 
-   int   Read(void *buf,int size);
+   int   Read(Buffer *buf,int size);
    int   Write(const void *buf,int size);
    int   Buffered();
    void  Close();
@@ -467,7 +471,7 @@ public:
    int	 StoreStatus();
 
    int	 Do();
-   void  Disconnect();
+   void  DisconnectLL();
    void  DisconnectNow();
 
    void	 SetFlag(int flag,bool val);
@@ -505,7 +509,7 @@ public:
 	 copy_protect=prot;
 	 copy_ssl_connect=sscn;
 	 retries=rnum;
-	 try_time=tt;
+	 SetTryTime(tt);
       }
    bool SetCopyAddress(const Ftp *o)
       {
