@@ -248,6 +248,7 @@ static ResType lftp_vars[] = {
    {"ftp:retry-530-anonymous",	 "Login incorrect",ResMgr::ERegExpValidate,0},
    {"hftp:cache",		 "yes",   ResMgr::BoolValidate,0},
    {"hftp:cache-control",	 "",	  0,0},
+   {"hftp:decode",		 "no",	  ResMgr::BoolValidate,0},
    {"hftp:proxy",		 "",	  HttpProxyValidate,0},
    {"hftp:use-authorization",	 "yes",   ResMgr::BoolValidate,0},
    {"hftp:use-head",		 "yes",   ResMgr::BoolValidate,0},
@@ -258,9 +259,11 @@ static ResType lftp_vars[] = {
    {"http:accept",		 "*/*",   0,0},
    {"http:accept-language",	 "",	  0,0},
    {"http:accept-charset",	 "",	  0,0},
+   {"http:accept-encoding",	 "",	  0,0},
    {"http:authorization",	 "",	  0,0},
    {"http:cache",		 "yes",   ResMgr::BoolValidate,0},
    {"http:cache-control",	 "",	  0,0},
+   {"http:decode",		 "yes",	  ResMgr::BoolValidate,0},
    {"http:proxy",		 "",	  HttpProxyValidate,0},
    {"http:use-mkcol",		 "yes",   ResMgr::BoolValidate,0},
    {"http:use-propfind",	 "no",    ResMgr::BoolValidate,0},
@@ -347,6 +350,7 @@ static ResType lftp_vars[] = {
    {"xfer:log",			 "yes",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:auto-rename",		 "no",	  ResMgr::BoolValidate,ResMgr::NoClosure},
    {"xfer:log-file",              "",     ResMgr::FileCreatable,ResMgr::NoClosure},
+   {"xfer:max-log-size",	 "1M",	  ResMgr::UNumberValidate,ResMgr::NoClosure},
 
 #if USE_SSL
    {"ssl:ca-file",		 "",	  ResMgr::FileReadable,ResMgr::NoClosure},
@@ -369,13 +373,12 @@ static ResDecls lftp_vars_register(lftp_vars);
 # include <langinfo.h>
 #endif
 
-bool ResMgr::class_inited;
-void ResMgr::ClassInit()
+void ResType::ClassInit()
 {
-   if(class_inited)
+   if(types_by_name)
       return;
-   class_inited=true;
-   for(ResType *scan=type_chain; scan; scan=scan->next)
+   types_by_name=new xmap<ResType*>;
+   for(ResType *scan=types_by_name->each_begin(); scan; scan=types_by_name->each_next())
    {
       if(scan->defvalue && scan->val_valid)
       {
